@@ -5,6 +5,12 @@
     .. autoclass:: Citation
         :members:
 
+    .. autoclass:: SphinxReferenceInfo
+        :members:
+
+    .. autoclass:: SphinxReferenceText
+        :members:
+
     .. autoclass:: BibtexDomain
         :members:
 """
@@ -17,19 +23,31 @@ import docutils.frontend
 import docutils.nodes
 import docutils.parsers.rst
 import docutils.utils
+import pybtex_docutils
+import pybtex.plugin
+from pybtex.richtext import Tag
+from pybtex.style.template import FieldIsMissing
+from pybtex.style import FormattedEntry
+
+import sphinxcontrib.bibtex.plugin
 import sphinx.util
 import re
 
-from pybtex.plugin import find_plugin
-from sphinx.domains import Domain
+from sphinx.domains import Domain, ObjType
 from sphinx.errors import ExtensionError
+from sphinx.locale import _
 from sphinx.util.nodes import make_refnode
 
+from .roles import CiteRole
 from .bibfile import BibFile, normpath_filename, process_bibfile
+from .nodes import raw_latex
+from .style.referencing import (
+    BaseReferenceText, BaseReferenceStyle, format_references
+)
 
 if TYPE_CHECKING:
+    from pybtex.backends import BaseBackend
     from pybtex.database import Entry
-    from pybtex.style import FormattedEntry
     from pybtex.style.formatting import BaseStyle
     from sphinx.addnodes import pending_xref
     from sphinx.application import Sphinx
@@ -38,7 +56,6 @@ if TYPE_CHECKING:
     from .directives import BibliographyKey, BibliographyValue
     from .roles import CitationRef
 
-
 logger = sphinx.util.logging.getLogger(__name__)
 
 
@@ -181,13 +198,17 @@ class _FilterVisitor(ast.NodeVisitor):
 
 
 def get_docnames(env):
-    """Ged document names in order."""
+    """Get document names in order."""
     rel = env.collect_relations()
     docname = env.config.master_doc
+    docnames = set()
     while docname is not None:
+        docnames.add(docname)
         yield docname
         parent, prevdoc, nextdoc = rel[docname]
         docname = nextdoc
+    for docname in sorted(env.found_docs - docnames):
+        yield docname
 
 
 class Citation(NamedTuple):
@@ -195,10 +216,49 @@ class Citation(NamedTuple):
     citation_id: str                     #: Unique id of this citation.
     bibliography_key: "BibliographyKey"  #: Key of its bibliography directive.
     key: str                             #: Key (with prefix).
-    label: str                           #: Label (with prefix).
+    entry: "Entry"                       #: Entry from pybtex.
     formatted_entry: "FormattedEntry"    #: Entry as formatted by pybtex.
 
 
+class SphinxReferenceInfo(NamedTuple):
+    """Tuple containing reference info to enable sphinx to resolve a reference
+    to a citation.
+    """
+    builder: "Builder"  #: The Sphinx builder.
+    fromdocname: str    #: Document name of the citation reference.
+    todocname: str      #: Document name of the bibliography.
+    citation_id: str    #: Unique id of the citation within the bibliography.
+
+
+class SphinxReferenceText(BaseReferenceText[SphinxReferenceInfo]):
+    """Pybtex rich text class for citation references with the docutils
+    backend, for use with :class:`SphinxReferenceInfo`.
+    """
+
+    def render(self, backend: "BaseBackend"):
+        assert isinstance(backend, pybtex_docutils.Backend), \
+               "SphinxReferenceText only supports the docutils backend"
+        info = self.info[0]
+        if info.builder.name == 'latex':
+            key = f'cite.{info.todocname}:{info.citation_id}'
+            return (
+                [raw_latex(f'\\hyperlink{{{key}}}{{')]
+                + super().render(backend)
+                + [raw_latex('}')]
+            )
+        else:
+            children = super().render(backend)
+            # make_refnode only takes a single child
+            refnode = make_refnode(
+                info.builder,
+                info.fromdocname,
+                info.todocname,
+                info.citation_id,
+                children[0])
+            refnode.extend(children[1:])  # type: ignore
+            return [refnode]
+
+
 def env_updated(app: "Sphinx", env: "BuildEnvironment") -> Iterable[str]:
     domain = cast(BibtexDomain, env.get_domain('cite'))
     return domain.env_updated()
@@ -218,6 +278,8 @@ class BibtexDomain(Domain):
         citations=[],
         citation_refs=[],
     )
+    backend = pybtex_docutils.Backend()
+    reference_style: BaseReferenceStyle
 
     @property
     def bibfiles(self) -> Dict[str, BibFile]:
@@ -250,6 +312,18 @@ class BibtexDomain(Domain):
         return self.data['citation_refs']
 
     def __init__(self, env: "BuildEnvironment"):
+        # set up referencing style
+        style = sphinxcontrib.bibtex.plugin.find_plugin(
+                'sphinxcontrib.bibtex.style.referencing',
+                env.app.config.bibtex_reference_style)
+        self.reference_style = style()
+        # set up object types and roles for referencing style
+        role_names = self.reference_style.role_names()
+        self.object_types = dict(
+            citation=ObjType(_('citation'), *role_names, searchprio=-1),
+        )
+        self.roles = dict((name, CiteRole()) for name in role_names)
+        # initialize the domain
         super().__init__(env)
         # connect env-updated
         env.app.connect('env-updated', env_updated)
@@ -273,8 +347,7 @@ class BibtexDomain(Domain):
                 document = docutils.utils.new_document(
                     "%s_header" % directive, settings)
                 parser.parse(header, document)
-                if len(document) > 0:
-                    self.data["%s_header" % directive] = document[0]
+                self.data["%s_header" % directive] = document[0]
 
     def clear_doc(self, docname: str) -> None:
         self.data['citations'] = [
@@ -308,10 +381,9 @@ class BibtexDomain(Domain):
         used_keys: Set[str] = set()
         used_labels: Dict[str, str] = {}
         for bibliography_key, bibliography in self.bibliographies.items():
-            for formatted_entry in self.get_formatted_entries(
+            for entry, formatted_entry in self.get_formatted_entries(
                     bibliography_key, docnames):
                 key = bibliography.keyprefix + formatted_entry.key
-                label = bibliography.labelprefix + formatted_entry.label
                 if bibliography.list_ == 'citation' and key in used_keys:
                     logger.warning(
                         'duplicate citation for key "%s"' % key,
@@ -320,19 +392,20 @@ class BibtexDomain(Domain):
                     citation_id=bibliography.citation_nodes[key]['ids'][0],
                     bibliography_key=bibliography_key,
                     key=key,
-                    label=label,
+                    entry=entry,
                     formatted_entry=formatted_entry,
                 ))
                 if bibliography.list_ == 'citation':
                     used_keys.add(key)
-                    if label not in used_labels:
-                        used_labels[label] = key
-                    elif used_labels[label] != key:
+                    if formatted_entry.label not in used_labels:
+                        used_labels[formatted_entry.label] = key
+                    elif used_labels[formatted_entry.label] != key:
                         # if used_label[label] == key then already
                         # duplicate key warning
                         logger.warning(
                             'duplicate label "%s" for keys "%s" and "%s"' % (
-                                label, used_labels[label], key),
+                                formatted_entry.label,
+                                used_labels[formatted_entry.label], key),
                             location=(bibliography_key.docname,
                                       bibliography.line))
         return []  # expects list of updated docnames
@@ -343,43 +416,46 @@ class BibtexDomain(Domain):
                      ) -> docutils.nodes.Element:
         """Replace node by list of citation references (one for each key)."""
         keys = [key.strip() for key in target.split(',')]
-        if builder.name != 'latex':
-            citations_node = docutils.nodes.inline(rawsource=target, text='[')
-        else:
-            citations_node = docutils.nodes.inline(rawsource=target, text='')
-        # map citation keys that can be resolved to their citation data
-        citations = {
+        citations: Dict[str, Citation] = {
             cit.key: cit for cit in self.citations
             if cit.key in keys
             and self.bibliographies[cit.bibliography_key].list_ == 'citation'}
-        for i, key in enumerate(keys):
-            try:
-                citation = citations[key]
-            except KeyError:
-                # TODO can handle missing reference warning using the domain
+        for key in keys:
+            if key not in citations:
                 logger.warning('could not find bibtex key "%s"' % key,
                                location=node)
-                citations_node += docutils.nodes.inline('', key)
-                continue
-            refcontnode = docutils.nodes.inline('', citation.label)
-            if builder.name == 'latex':
-                # latex builder needs a citation_reference
-                refnode = docutils.nodes.citation_reference(
-                    '', refcontnode,
-                    docname=citation.bibliography_key.docname,
-                    refname=citation.citation_id)
-            else:
-                # other builders can use general reference node
-                refnode = make_refnode(
-                    builder, fromdocname,
-                    citation.bibliography_key.docname,
-                    citation.citation_id, refcontnode)
-            citations_node += refnode
-            if i != len(keys) - 1 and builder.name != 'latex':
-                citations_node += docutils.nodes.Text(',')
-        if builder.name != 'latex':
-            citations_node += docutils.nodes.Text(']')
-        return citations_node
+        references = [
+            (citation.entry, citation.formatted_entry, SphinxReferenceInfo(
+                builder=builder,
+                fromdocname=fromdocname,
+                todocname=citation.bibliography_key.docname,
+                citation_id=citation.citation_id))
+            for citation in citations.values()]
+        formatted_references = \
+            format_references(
+                self.reference_style, SphinxReferenceText, typ, references)
+        result_node = docutils.nodes.inline(rawsource=target)
+        result_node += formatted_references.render(self.backend)
+        return result_node
+
+    def resolve_any_xref(self, env: "BuildEnvironment", fromdocname: str,
+                         builder: "Builder", target: str,
+                         node: "pending_xref", contnode: docutils.nodes.Element
+                         ) -> List[Tuple[str, docutils.nodes.Element]]:
+        """Replace node by list of citation references (one for each key),
+        provided that the target has citation keys.
+        """
+        keys = [key.strip() for key in target.split(',')]
+        citations: Set[str] = {
+            cit.key for cit in self.citations
+            if cit.key in keys
+            and self.bibliographies[cit.bibliography_key].list_ == 'citation'}
+        if any(key in citations for key in keys):
+            result_node = self.resolve_xref(
+                env, fromdocname, builder, 'p', target, node, contnode)
+            return [('p', result_node)]
+        else:
+            return []
 
     def get_all_cited_keys(self, docnames):
         """Yield all citation keys for given *docnames* in order, then
@@ -447,13 +523,30 @@ class BibtexDomain(Domain):
 
     def get_formatted_entries(
             self, bibliography_key: "BibliographyKey", docnames: List[str]
-            ) -> Iterable["FormattedEntry"]:
+            ) -> Iterable[Tuple["Entry", "FormattedEntry"]]:
         """Get sorted bibliography entries along with their pybtex labels,
         with additional sorting and formatting applied from the pybtex style.
         """
         bibliography = self.bibliographies[bibliography_key]
         entries = dict(
             self.get_sorted_entries(bibliography_key, docnames))
-        style = cast("BaseStyle", find_plugin(
+        style = cast("BaseStyle", pybtex.plugin.find_plugin(
             'pybtex.style.formatting', bibliography.style)())
-        return style.format_entries(entries.values())
+        sorted_entries = style.sort(entries.values())
+        labels = style.format_labels(sorted_entries)
+        for label, entry in zip(labels, sorted_entries):
+            try:
+                yield (
+                    entry,
+                    style.format_entry(
+                        bibliography.labelprefix + label, entry),
+                )
+            except FieldIsMissing as exc:
+                logger.warning(
+                    str(exc),
+                    location=(bibliography_key.docname, bibliography.line))
+                yield(
+                    entry,
+                    FormattedEntry(entry.key, Tag('b', str(exc)),
+                                   bibliography.labelprefix + label)
+                )
